﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SearchDemo
{
    public partial class FormSearchDemo : Form
    {
        public FormSearchDemo()
        {
            InitializeComponent();

            // Vychozi data.
            dataGridView1.Rows.Add(50);
            for (int Row = 0; Row < 50; Row++)
            {
                dataGridView1.Rows[Row].Cells["KodProduktu"].Value = "PN" + (Row + 1).ToString();
                dataGridView1.Rows[Row].Cells["NazevProduktu"].Value = "Bezva produkt číslo " + (Row + 1).ToString();
                dataGridView1.Rows[Row].Cells["Cena"].Value = Math.Pow(Row + 2, 2);
            }
        }

        /// <summary>Prevede hodnotu zadanou ve sloupci Cena na typ
        /// Double, ktera bude ulozena v DataGridViewCell.Value.
        /// </summary>
        private void dataGridView1_CellParsing(object sender, DataGridViewCellParsingEventArgs e)
        {
            if (e.ColumnIndex > -1 && e.RowIndex > -1 && e.Value != null)
            {
                // Jmeno sloupce, ve kterem je editovana bunka.
                string columnName = dataGridView1.Columns[e.ColumnIndex].Name;

                if (columnName == "Cena")
                {
                    try
                    {
                        // Prevedeni zadaneho textu na Double.
                        string number = e.Value.ToString();
                        number = number.Replace(',', '.');
                        e.Value = Double.Parse(number);
                        e.ParsingApplied = true;
                    }
                    catch (FormatException)
                    {
                        // Konverze selhala, ulozi se 0.
                        e.Value = 0.0;
                        e.ParsingApplied = true;
                    }
                }
            }
        }

        /// <summary>Porovna data ze sloupce Cena jako cislo.
        /// Ostatni sloupce se porovnavaji jako text.
        /// </summary>
        private void dataGridView1_SortCompare(object sender, DataGridViewSortCompareEventArgs e)
        {
            if (e.Column.Name == "Cena")
            {
                double d1, d2;
                if (!Double.TryParse(e.CellValue1 == null ? "0" : e.CellValue1.ToString(), out d1))
                    d1 = 0.0;
                if (!Double.TryParse(e.CellValue2 == null ? "0" : e.CellValue2.ToString(), out d2))
                    d2 = 0.0;

                if (d1 > d2)
                    e.SortResult = 1;
                else if (d1 < d2)
                    e.SortResult = -1;
                else
                    e.SortResult = 0;

                e.Handled = true;
            }
        }

        /// <summary>Zobrazi okno pro hledani po stisku tlacitka Hledat.
        /// </summary>
        private void buttonSearch_Click(object sender, EventArgs e)
        {
            ShowSearchTextForm();
        }

        /// <summary>Zachyti stisk klaves Ctrl+F a zobrazi okno pro hledani.
        /// </summary>
        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (keyData == (Keys.Control | Keys.F))
            {
                ShowSearchTextForm();
                return true;
            }

            return base.ProcessCmdKey(ref msg, keyData);
        }

        /// <summary>Zkontroluje, jestli jiz neni okno pro hledani zobrazeno
        /// a pokud neni, tak ho vytvori a nastavi delegaty pro hledani
        /// v DataGridView.
        /// </summary>
        private void ShowSearchTextForm()
        {
            // Projde okna ktera vlastnime
            // a pokud je mezi nimi trida FormSearchText
            // pouze ji zobrazi a nastavi focus.
            foreach (Form ownedForm in OwnedForms)
            {
                if (ownedForm is FormSearchText)
                {
                    ownedForm.Show();
                    ownedForm.Focus();
                    return;
                }
            }

            // Vytvoreni nove instance FormSearchText a nastaveni
            // metod, pro hledani v datech.
            FormSearchText searchForm = new FormSearchText();
            
            // Prida novou tridu do seznamu nami vlastnenych oken.
            AddOwnedForm(searchForm);
            searchForm.Owner = this;
            
            // Delegat spojeny s nasim novym oknem pro hledani.
            // Tento kod je volany pri stisku tlacitka Hledat
            // ve tride FormSearchText.
            searchForm.delTextSearch = (searchText, caseSensitive) =>
            {
                // Pokud nerozlisujeme velikost pismen, prevedeme hledany
                // text na mala pismena.
                if (!caseSensitive)
                    searchText = searchText.ToLower();

                // Porovani obsahu bunky s hledanym textem. Data ve sloupci Cena
                // se prevadi na Double a porovnavaji se s hledanou hodnotou,
                // ktera je take prevedena na Double.
                // Tento kod je volany pro kazdou DataGridViewCell
                // z nasledujiciho dotazu.
                Func<DataGridViewCell, bool, string, bool> CellValueMatch = (cell, cSensitive, sText) =>
                {
                    if (cell.OwningColumn.Name == "Cena")
                    {
                        try
                        {
                            if (Convert.ToDouble(sText) == Convert.ToDouble(cell.Value))
                                return true;
                            else
                                return false;
                        }
                        catch (FormatException)
                        {
                            return false;
                        }
                    }
                    else
                    {
                        string cellValue = cell.Value.ToString();
                        if (!cSensitive)
                            cellValue = cellValue.ToLower();

                        if (cellValue.IndexOf(sText) > -1)
                            return true;
                        else
                            return false;
                    }
                };

                // Dotaz na data v DataGridView. Prohleda kazdou bunku v tabulce
                // krome radku pro novy zaznam a vrati 1. DataGridViewCell,
                // ktera obsahuje hledany text.
                // Pokud hledani nic nenajde, vyhodi vyjimku InvalidOperationException.
                try
                {
                    dataGridView1.CurrentCell = (from dgRow in dataGridView1.Rows.Cast<DataGridViewRow>()
                                                 where !dgRow.IsNewRow
                                                 from dgCell in dgRow.Cells.Cast<DataGridViewCell>()
                                                 where dgCell.Value != null
                                                 && CellValueMatch(dgCell, caseSensitive, searchText)
                                                 select dgCell).First();
                }
                catch (InvalidOperationException)
                {
                    MessageBox.Show("Hledaný text nebyl nalezen.", Text);
                }
            };

            // Zobrazeni okna pro hledani.
            searchForm.Show();
        }
    }
}
